

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////



#ifndef __HELPER_H__
#define __HELPER_H__

#include <string>
#include <vector>

#include "plink.h"

template<class T>
inline const T SQR(const T a) {return a*a;}
 
template<class T>
inline const T MAX(const T &a, const T &b)
        {return b > a ? (b) : (a);}
 
template<class T>
inline const T MIN(const T &a, const T &b)
        {return b < a ? (b) : (a);}
 
template<class T>
inline const T SIGN(const T &a, const T &b)
        {return b >= 0 ? (a >= 0 ? a : -a) : (a >= 0 ? -a : a);}
 
template<class T>
inline void SWAP(T &a, T &b)
        {T dum=a; a=b; b=dum;}


class Plink;
class Individual;
class CSNP;

using namespace std;

void NoMem();

class CArgs
{
 public:
  CArgs(int,char**);

  
  int count()
    { return n; }
  
  bool any()
    { return n > 1 ? true : false; }
  
  void fromScript(string);
  bool find(string);
  string value(string);
  int value_int(string);
  double value_double(string);
  
  void check_unused_options(Plink &);
  
  vector<string> value(string,int);
  
  vector<string> a;
  
 private:
  int n;
  vector<bool> parsed;
};


void error(string);
void shutdown();
bool readString(FILE *,string &);
bool realnum(double);

void sizeMatrix(vector<vector<double> > &, int,int);
void multMatrix(vector<vector<double> > & a,
		vector<vector<double> > & b,
		vector<vector<double> > & c);


int getInt(string,string);
double getDouble(string,string);

void permute(vector<long int>&);
void permute(vector<int>&);

vector<double> FDR_BH(vector<double>&);

void affCoding(Plink &);
void removeMissingPhenotypes(Plink & );

string genotype(Plink &, int i, int l);

long double factorial(int);


int getMarkerChromosome(Plink &,string);
int getMarkerNumber(Plink &,string);
string leftWindowEdge(Plink & P, int bp, int chr);
string rightWindowEdge(Plink & P, int bp, int chr);
vector<int> getChromosomeMarkerRange(Plink &, int);
vector<int> getChromosomeRange(Plink &);
vector<int> getWindowRange(Plink &P, int);

double SNPHWE(int obs_hets, int obs_hom1, int obs_hom2);

string int2str(int);
string dbl2str(double);

double normdist(double);
double ltqnorm(double);
std::string itoa(int, int);


void checkFileExists(string);

void defineDogChromosomes();
void defineHumanChromosomes();

double chiprobP(double df, double chi);
double gammp(double a, double x);
void gser(double *gamser, double a, double x, double *gln);
void gcf(double *gammcf, double a, double x, double *gln);
double gammln(double xx);

void lubksb(vector<vector<double> > &a, vector<int> &indx, vector<double> &b);
void ludcmp(vector<vector<double> > &a, vector<int> &indx, double &d);
vector< vector<double> > inverse(vector< vector<double> > & m );

vector<double> eigenvalues(vector<vector<double> > & a);
void tred2(vector<vector<double> >&,vector<double> &,vector<double> &);
void tqli(vector<double> &d, vector<double>&e, vector<vector<double> > &z);
vector< vector<double> > svd_inverse(vector< vector<double> > & );
void svdcmp(vector<vector<double> > &, 
	    vector<double> &, 
	    vector<vector<double> > &);
void svbksb(vector<vector<double> > &u,
	    vector<double> &w,
	    vector<vector<double> > &v,
	    vector<double> &b, 
	    vector<double> &x);
vector<vector<double> > msqrt(vector<vector<double> > & u);

void svdvar(vector<vector<double> > & v,
	    vector<double> & w,
	    vector<vector<double> > & cvm);

vector<bool> vif_prune(vector<vector<double> > , double threshold);
void calcSetMeanVariance(vector<CSNP*> & pSNP,
			 vector<double> & mean,
			 vector<vector<double> > & variance,
			 vector<Individual*> & sample);

double pythag(const double a, const double b);

double betacf(const double a, const double b, const double x);
double betai(const double a, const double b, const double x);
double pF(const double F, const int df1, const int df2);





#endif
