

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////


#include <iostream>
#include "plink.h"
#include "options.h"

using namespace std;

vector<Z> Plink::calcLocusIBD(Individual * p1, Individual * p2, Z I)
{

  // I is type Z, contains genomewide IBS estimates

  // Genome-wide IBD for this pair 
  vector<Z> ZL(nl);

  // Locus counter
  int l = 0;
  

  //////////////////////////////
  // All SNPs in the scan region

  for (int l2=par::run_start; l2<=par::run_end; l2++)
    {
      

      /////////////////////
      // Allele frequencies

      double p = locus[l2]->freq;
      double q = 1 - p;


      ///////////////////////////////
      // 1. Precalculate P(G1,G2|IBD)
      //    (true genotype given IBD)


	// P( G1, G2 | IBD=0 )

      double P_tru_AA_AA_0 = p*p*p*p;
      double P_tru_AA_AB_0 = 4*p*p*p*q;
      double P_tru_AA_BB_0 = 2*p*p*q*q;

      double P_tru_AB_AA_0 = P_tru_AA_AB_0; 
      double P_tru_AB_AB_0 = 4*p*p*q*q;
      double P_tru_AB_BB_0 = 4*p*q*q*q;

      double P_tru_BB_AA_0 = P_tru_AA_BB_0; 
      double P_tru_BB_AB_0 = P_tru_AB_BB_0;
      double P_tru_BB_BB_0 = q*q*q*q;


	// P( G1, G2 | IBD=1 )

      double P_tru_AA_AA_1 = p*p*p;
      double P_tru_AA_AB_1 = 2*p*p*q;
      double P_tru_AA_BB_1 = 0;

      double P_tru_AB_AA_1 = P_tru_AA_AB_1; 
      double P_tru_AB_AB_1 = p*q;
      double P_tru_AB_BB_1 = 2*p*q*q;

      double P_tru_BB_AA_1 = P_tru_AA_BB_1; 
      double P_tru_BB_AB_1 = P_tru_AB_BB_1;
      double P_tru_BB_BB_1 = q*q*q;


	// P( G1, G2 | IBD=2 )

      double P_tru_AA_AA_2 = p*p;
      double P_tru_AA_AB_2 = 0;
      double P_tru_AA_BB_2 = 0;

      double P_tru_AB_AA_2 = 0;
      double P_tru_AB_AB_2 = 2*p*q;
      double P_tru_AB_BB_2 = 0;

      double P_tru_BB_AA_2 = 0;
      double P_tru_BB_AB_2 = 0;
      double P_tru_BB_BB_2 = q*q;


      

      // Considering this specific pair of individuals
      
      int g1 = 1;   // 12 
      if ( (!p1->one[l2])  && (!p1->two[l2]) ) g1 = 0;
      else if (   p1->one[l2]   &&   p1->two[l2]  ) g1 = 2;
      else if (   p1->one[l2]   && (!p1->two[l2]) ) g1 = -9;
      
      int g2 = 1;   // 12 
      if ( (!p2->one[l2]) && (!p2->two[l2]) ) g2 = 0;
      else if (   p2->one[l2]  &&  p2->two[l2]   ) g2 = 2;
      else if (   p2->one[l2]  && (!p2->two[l2]) ) g2 = -9;


      
      ////////////////////////////////////////////
      // 2. Allow for possible genotyping error
      //    sum_{all possible true genotypes} P(observed G|true G) P(true G |IBD)

//       double e = 0.005;
//       double f = 0.001;

//       double ER_AA_AA__AA_AA = 1- 2e - 2f - 2e*f - e*e - f*f;
//       double ER_AA_AB__AA_AA = e;
//       double ER_AA_BB__AA_AA = f;
	
//       double ER_AB_AA__AA_AA = e;
//       double ER_AB_AB__AA_AA = e*e;
//       double ER_AB_BB__AA_AA = e*f;
      
//       double ER_BB_AA__AA_AA = f;
//       double ER_BB_AB__AA_AA = e*f;
//       double ER_BB_BB__AA_AA = f*f;

      
//       double ER_AA_AA__AA_AB = e;
//       double ER_AA_AB__AA_AB = 1 - 3*e - 2*e*e - 2*e*f - f;
//       double ER_AA_BB__AA_AB = e;

//       double ER_AB_AA__AA_AB = e*e;
//       double ER_AB_AB__AA_AB = e;
//       double ER_AB_BB__AA_AB = e*e;
      
//       double ER_BB_AA__AA_AB = f*e;
//       double ER_BB_AB__AA_AB = f;
//       double ER_BB_BB__AA_AB = f*e;



      // P( G1, G2 | IBD=0 )
      
      double P_AA_AA_0 = P_tru_AA_AA_0;
      double P_AA_AB_0 = P_tru_AA_AB_0;
      double P_AA_BB_0 = P_tru_AA_BB_0;

      double P_AB_AA_0 = P_tru_AB_AA_0;
      double P_AB_AB_0 = P_tru_AB_AB_0;
      double P_AB_BB_0 = P_tru_AB_BB_0;

      double P_BB_AA_0 = P_tru_BB_AA_0;
      double P_BB_AB_0 = P_tru_BB_AB_0;
      double P_BB_BB_0 = P_tru_BB_BB_0;


	// P( G1, G2 | IBD=1 )

      double P_AA_AA_1 = P_tru_AA_AA_1;
      double P_AA_AB_1 = P_tru_AA_AB_1;
      double P_AA_BB_1 = P_tru_AA_BB_1;

      double P_AB_AA_1 = P_tru_AB_BB_1;
      double P_AB_AB_1 = P_tru_AB_BB_1;
      double P_AB_BB_1 = P_tru_AB_BB_1;

      double P_BB_AA_1 = P_tru_BB_BB_1;
      double P_BB_AB_1 = P_tru_BB_BB_1;
      double P_BB_BB_1 = P_tru_BB_BB_1;


	// P( G1, G2 | IBD=2 )

      double P_AA_AA_2 = P_tru_AA_AA_2;
      double P_AA_AB_2 = P_tru_AA_AB_2;
      double P_AA_BB_2 = P_tru_AA_BB_2;

      double P_AB_AA_2 = P_tru_AB_AA_2;
      double P_AB_AB_2 = P_tru_AB_AB_2;
      double P_AB_BB_2 = P_tru_AB_BB_2;

      double P_BB_AA_2 = P_tru_BB_AA_2;
      double P_BB_AB_2 = P_tru_BB_AB_2;
      double P_BB_BB_2 = P_tru_BB_BB_2;

      
      
      // Sum over all possible true genotypes P(Observed G | True G) P(True G |IBD = 1)


      ///////////////////////
      // 3. Account for LD
      



      //////////////////////////////////
      // 4. Calculate P(IBD | G1, G2)
      // 
      //  P (IBD|G1,G2) =  P(G1,G2|IBD)P(IBD) 
      //         /  SUM_IBD { P(G1,G2|IBD)P(IBD) } 
      


      /////////////
      // 00 x 00 

      if ( g1==-9 || g2==-9 ) 		    
	{
	  ZL[l] = I;
	}  


  
      ///////////////////
      // AA x AA (IBS 2) 
      
      if ( g1==0 && g2==0 ) 
	{
	  // P(IBD=0|AA,AA) 
	  ZL[l].z0 = ( P_AA_AA_0 * I.z0 ) / 
	    ( P_AA_AA_0*I.z0 + P_AA_AA_1*I.z1 + P_AA_AA_2*I.z2 ); 
	  
	  // P(IBD=1|AA,AA) 
	  ZL[l].z1 = ( P_AA_AA_1 * I.z1 ) / 
	    ( P_AA_AA_0*I.z0 + P_AA_AA_1*I.z1 + P_AA_AA_2*I.z2 );
	  
	  // P(IBD=2|AA,AA) 
	  ZL[l].z2 = ( P_AA_AA_2 * I.z2 ) / 
	    ( P_AA_AA_0*I.z0 + P_AA_AA_1*I.z1 + P_AA_AA_2*I.z2 );
	  
	}
      

      ////////////////////
      //  AA x AB (IBS 1) 
      
      if ( ( g1==0 && g2==1 ) || 
	   ( g1==1 && g2==0 ) ) 
	{
	  // P(IBD=0|11,12) 
	  ZL[l].z0 = ( P_AA_AB_0 * I.z0 ) / 
	    ( P_AA_AB_0*I.z0 + P_AA_AB_1*I.z1 ); 
	  
	  // P(IBD=1|11,12) 
	  ZL[l].z1 = ( P_AA_AB_1 * I.z1 ) / 
	    ( P_AA_AB_0*I.z0 + P_AA_AB_1*I.z1 );
	  
	  // P(IBD=2|11,12) 
	  ZL[l].z2 = 0;
	  
	}
      
            
      
      ////////////////////
      // AA x BB (IBS 0) 
      
      if ( ( g1==0 && g2==2 ) || 
	   ( g1==2 && g2==0 ) ) 
	{
	  // P(IBD=0|11,22) 
	  ZL[l].z0 = 1; 
	  
	  // P(IBD=1|11,11) 
	  ZL[l].z1 = 0;
	  
	  // P(IBD=2|11,11) 
	  ZL[l].z2 = 0;
	  
	}
      
      
      
      /////////////////
      // AB x AB (IBS 2)
      
      if ( g1==1 && g2==1 ) 
	{
	  // P(IBD=0|12,12) 
	  ZL[l].z0 = ( P_AB_AB_0 * I.z0 ) / 
	    ( P_AB_AB_0*I.z0 + P_AB_AB_1*I.z1 + P_AB_AB_2*I.z2 ); 
	  
	  // P(IBD=1|12,12) 
	  ZL[l].z1 = ( P_AB_AB_1 * I.z1 ) / 
	    ( P_AB_AB_0*I.z0 + P_AB_AB_1*I.z1 + P_AB_AB_2*I.z2 );
	  
	  // P(IBD=2|12,12) 
	  ZL[l].z2 = ( P_AB_AB_2 * I.z2 ) / 
	    ( P_AB_AB_0*I.z0 + P_AB_AB_1*I.z1 + P_AB_AB_2*I.z2 );
	  
	}
      
      
      ////////////////////
      // AB x BB  (IBS 1)
      
      if ( ( g1==1 && g2==2 ) || 
	   ( g1==2 && g2==1)  ) 
	{
	  // P(IBD=0|AB,BB) 
	  ZL[l].z0 = ( P_AB_BB_0 * I.z0 ) / 
	    ( P_AB_BB_0*I.z0 + P_AB_BB_1*I.z1 ); 
	  
	  // P(IBD=1|AB,BB) 
	  ZL[l].z1 = ( P_AB_BB_1 * I.z1 ) / 
	    ( P_AB_BB_0*I.z0 + P_AB_BB_1*I.z1 );
	  
	  // P(IBD=2|AB,BB) 
	  ZL[l].z2 = 0;
	  
	}
      

      
      //////////////////
      // BB x BB (IBS 2)
      
      if ( g1==2 && g2==2 ) 
	{
	  // P(IBD=0|BB,BB) 
	  ZL[l].z0 = ( P_BB_BB_0 * I.z0 ) / 
	    ( P_BB_BB_0*I.z0 + P_BB_BB_1*I.z1 + P_BB_BB_2*I.z2 ); 
	  
	  // P(IBD=1|BB,BB) 
	  ZL[l].z1 = ( P_BB_BB_1 * I.z1 ) / 
	    ( P_BB_BB_0*I.z0 + P_BB_BB_1*I.z1 + P_BB_BB_2*I.z2 );
	  
	  // P(IBD=2|BB,BB) 
	  ZL[l].z2 = ( P_BB_BB_2 * I.z2 ) / 
	    ( P_BB_BB_0*I.z0 + P_BB_BB_1*I.z1 + P_BB_BB_2*I.z2 );
	  
	}
      		   
      

      //////////////////////////////////////////////////////////////
      // Fudge factor to allow for small chance of genotyping error

      if ( ZL[l].z1 < 0.0001 ) 
	{
	  ZL[l].z1 = 0.0001;
	  double S = ZL[l].z0 + ZL[l].z1 + ZL[l].z2;
	  ZL[l].z0 /= S;
	  ZL[l].z1 /= S;
	  ZL[l].z2 /= S;
	}
	   
      
      
      // Next locus
      l++;
    } 

  return ZL;
}
  

