

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////


#include <iostream>
#include <iomanip>
#include <map>

#include "plink.h"
#include "helper.h"
#include "options.h"
#include "linear.h"

void Plink::perm_testQTDT(Perm &)
{

  // Use individual-major coding
  if (par::SNP_major) 
    SNP2Ind();
  
  // Set up some necessary things
  
  vector<bool> dummy;
  string f = par::output_file_name + ".qfam";
  if (par::QFAM_within1) f += ".within";
  else if (par::QFAM_within2) f += ".parents";
  else if (par::QFAM_total) f += ".total";

  printLOG("\nWriting QFAM results to [ "+f+" ]\n");
  ofstream QOUT(f.c_str(),ios::out);
  QOUT.precision(4);

  // Run original QTDT

  vector_t orig = calcQTDT(true, QOUT, false, dummy, dummy, dummy);

  QOUT.close();


  // Permutation?

  if (!par::permute) return;

  
}

vector_t Plink::calcQTDT(bool print_results, 
			 ofstream & QOUT,
			 bool permute, 
			 vector<bool> & fA, 
			 vector<bool> & fB, 
			 vector<bool> & fP)
{
  
  vector<string> label;
  if (print_results)
    {
      if (par::QFAM_total)
	{
	  label.push_back("I");
	  label.push_back("G");
	}
      else 
	{
	  label.push_back("I");
	  label.push_back("B");
	  label.push_back("W");
	}
    }
  
  //////////////////////
  // Dependent variable
  // and 
  // familial clustering

  vector_t Y; 
  vector<int> C;
  vector<Individual*>::iterator person = sample.begin(); 
  map<string,int> famcnt;
  int fc=0;
  while ( person != sample.end() )
  {
    if ( ! (*person)->missing )
     {
       Y.push_back( (*person)->phenotype );
       map<string,int>::iterator fi = famcnt.find( (*person)->fid );
       if ( fi == famcnt.end() )
	 {
	   famcnt.insert( make_pair( (*person)->fid , fc ));
	   C.push_back(fc);            
	   fc++;
	 }
       else
	 {
	   C.push_back( fi->second );
	 }   
     }
    
    person++;  
  } 
  int ns = Y.size();
  
  printLOG(int2str(family.size())+" nuclear families in analysis\n");
  printLOG(int2str(famcnt.size())+" distinct family IDs found (clusters)\n");
  
  if ( family.size()<2 )
    error("Halting: not enough nuclear families for this analysis\n");
  
 /////////////////////////
 // Iterate over each SNP

 vector_t results;
 
 for (int l=0; l<nl_all; l++)
   {     
     
     // Form linear model
     
     LinearModel lm;
     
     vector<Individual*>::iterator gperson = sample.begin();
     
     vector_t X;
     vector<bool> include;
     vector<Individual*> tind;

     while ( gperson != sample.end() )
       {
	 
	 Individual * pperson = (*gperson)->pperson;
	 
	 if ( ! pperson->missing  )
	   {
	     double xi;
	     bool inc = true;

	     bool s1 = (*gperson)->one[l];
	     bool s2 = (*gperson)->two[l];	     
	     
	     if ( ! s1 ) 
	       {
		 if ( ! s2 ) 
		   xi = 1;
		 else 
		   xi = 0;
	       } 
	     else
	       {
		 if ( ! s2 ) 
		   inc = false;
		 else 
		   xi = -1;
	       }
	     
	     if (inc)
	       X.push_back(xi);
	     else
	       X.push_back(-9);

	     include.push_back(inc);
	     tind.push_back(*gperson);
	   }
	 
	 // Next individual
	 gperson++;  	 
       }
     


     ///////////////////////////////////////
     // Score between and within components

     vector_t B;
     vector_t W;
     
     vector_t X2;
     vector_t Y2;
     vector<int> C2;
     
     if ( ! par::QFAM_total )
       {

	 vector<Family*>::iterator f = family.begin();
          
	 // Construct family B score for this SNP
	 
	 while ( f != family.end() )
	   {
	     

	     if (par::verbose)
	       {
		 if ( ! (*f)->parents )
		   {
		     cout << "SIBSHIP " << (*f)->kid[0]->fid << " ";
		     for ( int k=0; k<(*f)->kid.size(); k++)
		       cout << (*f)->kid[k]->iid << " ";
		     cout << "\n";
		   }
		 else
		   {
		     cout << "W/ PARENTS " << (*f)->pat->fid << " ( ";
		     cout << (*f)->pat->iid << " x " << (*f)->mat->iid << " ) ";
		     for ( int k=0; k<(*f)->kid.size(); k++)
		       cout << (*f)->kid[k]->iid << " ";
		     cout << "\n";	     
		   }
	       }
	     
	     
	     double B = 0;
	     bool Bset = false;
	     
	     // Two genotyped parents?
	     if ( (*f)->parents )
	       {
		 
		 bool p1 = (*f)->pat->one[l];
		 bool p2 = (*f)->pat->two[l];
		 
		 bool m1 = (*f)->mat->one[l];
		 bool m2 = (*f)->mat->two[l];
		 
		 bool parents = true;
		 
		 if ( p1 && !p2 ) parents = false;
		 else if ( m1 && !m2 ) parents = false;
		 
		 // Use parental genotypes, if present for this SNP
		 if ( parents )
		   {
		     
		     if ( ! p1 )
		       {
			 if ( ! p2 ) 
			   B++;
		       }
		     else 
		       B--;
		     
		     
		     if ( ! m1 )
		       {
			 if ( ! m2 ) 
			   B++;
		       }
		     else 
		       B--;
		     
		     B /= 2;
		     Bset = true;
		   }
	       }
	     
	     if ( !Bset ) 
	       {
		 // Use sibling genotypes
		 int nsib = (*f)->kid.size();
		 int nsib2 = nsib;
		 for (int k=0; k<nsib; k++)
		   {
		     bool k1 = (*f)->kid[k]->one[l];
		     bool k2 = (*f)->kid[k]->two[l];
		     
		     if ( ! k1 ) 
		       {
			 if ( ! k2 )
			   B++;
		       }
		     else 
		       {
			 if ( ! k2 )
			   nsib2--;
			 else 
			   B--;
		       }		 
		   }
		 
		 if (nsib2==0)
		   B == -9;
		 else
		   B /= nsib2;
		 		 
	       }
	     
	     
	     if ( ! realnum(B) ) error("problemmo!\n");

	     // Store between family score
	     (*f)->B = B;

	     // Next family
	     f++;
	     
	   }
	 


	 vector<Individual*> tind2;
	 gperson = sample.begin();
	 int i=0;
	 
	 while ( gperson != sample.end() )
	   {
	     
	     Individual * pperson = (*gperson)->pperson;
	     
	     // Valid phenotype...
	     if ( ( ! pperson->missing ) )
	       {
		 // ... and genotype?
		 if ( include[i] )
		   {
		     // Are we modelling parental phenotypes?
		     if ( par::QFAM_total ||    // total association test
			  ( par::QFAM_within2 && (*gperson)->family->parents ) || // parenQTDT
			  ! (*gperson)->founder )  // not a founder
		       {
			 
			 // Between-family component
			 double b = (*gperson)->family->B;
			 
			 B.push_back( b );
			 
			 bool s1 = (*gperson)->one[l];
			 bool s2 = (*gperson)->two[l];	     
			 
			 if ( ! s1 ) 
			   {
			     if ( ! s2 ) 
			       W.push_back( 1 - b );
			     else 
			       W.push_back( 0 - b );
			   } 
			 else
			   {
			     W.push_back( - 1 - b );
			   }
			 
			 // Add phenotype and genotype for extracted sample
			 Y2.push_back(Y[i]);
			 X2.push_back(X[i]);
			 C2.push_back(C[i]);
			 tind2.push_back(tind[i]);
			 
		       }
		   }	     
		 
		 i++; // next non-missing person
	       }
	     
	     // Next person
	     gperson++;
	     
	   }
	 
	 if (par::verbose)
	   {
	     cout << "Y,X,B,W sizes = "
		  << Y2.size() << " "
		  << X2.size() << " "
		  << B.size() << " "
		  << W.size() << " "
		  << tind2.size() << "\n";
	     
	     for (int i=0; i<Y2.size(); i++)
	       cout << tind2[i]->fid << " "
		    << tind2[i]->iid << "\t"
		    << C2[i] << "\t"
		    << Y2[i] << "\t"
		    << X2[i] << "\t"
		    << B[i] << "\t"
		    << W[i] << "\n";
	     cout << "\n";
	   }

       	 
       } // end of B,W scoring 
     
     ////////////////////////
     // Perform QFAM analysis
     
     

     if (par::QFAM_total) 
       {
	 lm.setDependent(Y);
	 lm.setCluster(C);
	 lm.addIndependent(X);
       }
     else if (par::QFAM_within1 || par::QFAM_within2) 
       {
	 // Within-family QTDT, or parenQTDT
	 lm.setDependent(Y2);
	 lm.setCluster(C2);
	 lm.addIndependent(B);
	 lm.addIndependent(W);
       }

     
     //////////////
     // Covariates?
     
     if (0) // par::covariates...
       {
	 // 
       } 
     
     
     lm.fitLM();
     
     
     vector_t b = lm.getCoefs();
     vector_t var = lm.getVar();
     
     // Test statistic
     results.push_back( (b[1]*b[1] ) / var[1] );
     
     // Result?
     if (print_results)
       {
	 // Skip intercept
	 for (int i=1; i<b.size(); i++)		
	   {
	     double wald = ( b[i]*b[i] ) / var[i];
	     
	     QOUT << setw(par::pp_maxsnp) << locus[l]->name << " " 
		  << setw(4) << locus[l]->chr << " ";
	     QOUT << setw(4) << label[i] << " "
		  << setw(8) << b[i] << " "
		  << setw(8) << sqrt(var[i]) << " "
		  << setw(10) << wald << " "
		  << setw(10) << chiprobP(wald,1);
	     QOUT << "\n";
	   }
       }
     

     // Next SNP
   }
 
 return results;

}


