

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////


#include <iostream>
#include <sstream>

#include "plink.h"
#include "options.h"

// Requires wsock32.lib     WINDOWS
//          -lsocket -nsl   BSD

//#define  UNIX            // WIN/UNIX/SKIP


#ifndef SKIP

#include <stdio.h>
#include <stdlib.h>
#include <string> 
#include <unistd.h>

#ifdef WIN
#include <windows.h>
#endif

#ifdef UNIX
#include <sys/types.h>    // Needed for system defined identifiers.
#include <netinet/in.h>   // Needed for internet address structure.
#include <sys/socket.h>   // Needed for socket(), bind(), etc...
#include <arpa/inet.h>    // Needed for inet_ntoa()
#include <fcntl.h>
#include <netdb.h>
#include <signal.h>
#endif

#define  BUF_SIZE            4096     
#define  PORT_NUM              80     
#define  IP_ADDR    "132.183.161.22"  
#define  WWW_NAME   "pngu.mgh.harvard.edu"
#define  GET_STRING "GET /~purcell/plink/version.txt HTTP/1.1\nHost: pngu.mgh.harvard.edu\nConnection: close\n\n"

using namespace std;

extern string PVERSION;

#endif      // of SKIP 


void Plink::webcheck()
{

#ifdef SKIP
  printLOG("Web-check not implemented on this system...\n");
  return;

#else
  

  printLOG("Web-based version check ( --noweb to skip )\n");
  printLOG("Connecting to web... ");

  
#ifdef WIN
  WORD wVersionRequested = MAKEWORD(1,1);    // Stuff for WSA functions
  WSADATA wsaData;                           // Stuff for WSA functions
#endif

  
  unsigned int         server_s;             // Server socket descriptor
  struct sockaddr_in   server_addr;          // Server Internet address
  char                 out_buf[BUF_SIZE];    // Output buffer for GET request
  char                 in_buf[BUF_SIZE];     // Input buffer for response
  unsigned int         retcode;              // Return code
  unsigned int         i;                    // Loop counter
  
#ifdef WIN
  WSAStartup(wVersionRequested, &wsaData);
#endif

  // Create a socket
  server_s = socket(AF_INET, SOCK_STREAM, 0);
  
  // Fill-in the Web server socket's address information
  server_addr.sin_family = AF_INET;                 // Address family to use
  server_addr.sin_port = htons(PORT_NUM);           // Port num to use
  
  server_addr.sin_addr.s_addr = inet_addr(IP_ADDR); // IP address to use
  //server_addr.sin_addr = *((struct in_addr *)he->h_addr); 

  // Do a connect (connect() blocks)
  retcode = connect(server_s, (struct sockaddr *)&server_addr,
                    sizeof(server_addr));
  if (retcode != 0)
  {
    printLOG(" failed connection\n\n");
    return;
  }

  // Send a GET to the Web server
  strcpy(out_buf, GET_STRING);
  send(server_s, out_buf, strlen(out_buf), 0);

  // Receive from the Web server
  //  - The return code from recv() is the number of bytes received
  retcode = recv(server_s, in_buf, BUF_SIZE, 0);
  while ((retcode > 0) || (retcode == -1))
  {
    if (retcode == -1)
    {
      printLOG(" failed connection\n\n");
      return;
    }

    retcode = recv(server_s, in_buf, BUF_SIZE, 0);
  }
  

  string buf; 
  stringstream ss(in_buf); 
  vector<string> tokens; 
  while (ss >> buf)
    tokens.push_back(buf);

  bool print = false;
  bool print2 = false;
  bool version_okay = true;

  for (int i=0; i<tokens.size(); i++)
    {

      if (tokens[i]=="END") break;

      if (tokens[i]=="MESSAGE-ALL")
	{
	  print2=true;
	  continue;
	}

      // Display any other messages
      // Either conditional on old version (print)
      // or a broadcast to all users (print2)

      if ( ( print && !version_okay) || print2 ) 
	{
	  if (tokens[i]=="\\n")
	    printLOG("\n");
	  else
	    printLOG(tokens[i]+" ");
	}

      // Check version code
      if (tokens[i]=="PLINK-VERSION") 
	{
	  print=true;
	  if ( i < tokens.size() - 1) 
	    {
	      if (tokens[i+1] == PVERSION)
		printLOG(" OK, v"+PVERSION+" is current\n");
	      else
		{
		  printLOG("\n\n          *** UPDATE REQUIRED ***\n\n");
		  printLOG("\tThis version        : "+PVERSION+"\n");
		  printLOG("\tMost recent version : "+tokens[i+1]+"\n\n");
		  printLOG("Please upgrade your version of PLINK as soon as possible!\n"); 
		  printLOG("  (visit the above website for free download)\n\n");
		  version_okay=false;
		}

	      // Skip the version number
	      i++;
	    }
	}

    }

  // did we get the information we needed?
  if (!print) printLOG(" problem connecting to web\n");

  printLOG("\n");
  
  // Close all open sockets
#ifdef WIN
  closesocket(server_s);
#endif
#ifdef UNIX
  close(server_s);
#endif
  
#ifdef WIN
  WSACleanup();
#endif

#endif  // SKIP
}
